# -*- coding:utf-8 -*-
import logging
from utils import reformat_lm, get_file_list, get_object_meta
from client import Client
from baidubce import exception
from baidubce.services import bos
from bosftp_config import BosftpCfg
from utils import FifoBuffer
logger = logging.getLogger(__name__)

class Appender(object):
    def __init__(self, bucket_name, object_key):
        self.name = object_key
        self.buffer = FifoBuffer() 
        self.closed = False 
        try:
            response = Client().bos_client.get_object_meta_data(BosftpCfg().bucket, str(object_key))
        except exception.BceError as e:
            if "Not Found" in str(e.message):
                self.offset = 0
            else:
                logger.exception( "get_object_meta_data in getsize occurs an exception.bucket_name: {0}, object_key: {1}, err_msg: {2}".format(
                    str(self.bucket_name).encode("utf-8"), str(self.object_key).encode("utf-8"), e.message))
                raise IOError("failed to init appender") 
        else:
            self.offset = response.metadata.content_length 

        #self.appe_block_size = BosftpCfg().appe_block_size
    # file write interface
    def write(self, data):
        logger.info("APPE Recv data size: %d, file: %s" % (len(data), self.name))         

        # accumulate data to appe block size 
        self.buffer.write(data)
        return len(data)

    # file close interface
    def close(self):
        logger.info("Closing file: {0}".format(self.name))
        # append data 
        #if self.buffer.size() >= self.appe_block_size:
        try:
            if self.offset == 0:
                response = Client().bos_client.append_object_from_string(BosftpCfg().bucket, str(self.name), #offset=self.offset, 
                                   data=self.buffer.read(self.buffer.size()))
            else:
                response = Client().bos_client.append_object_from_string(BosftpCfg().bucket, str(self.name), 
                                           offset=self.offset, 
                                           data=self.buffer.read(self.buffer.size()))
        except exception.BceError as e:
            logger.error("sdk append object %s failed, msg: %s" % (self.name, str(e.message)))
            raise IOError("sdk append object %s failed, msg: %s" % (self.name, str(e.message)))
        #else:
        #    self.offset = response.metadata.bce_next_append_offset 
        logger.info("Append object:%s success, offset:%d" % (self.name, int(self.offset)))
        self.closed = True 
