# -*- coding:utf-8 -*-

from multiprocessing.pool import ThreadPool
from bosftp_config import BosftpCfg
import threading
import logging
import time
logger = logging.getLogger(__name__)


class BosThreadPool(object):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(BosThreadPool, "_instance"):
            with BosThreadPool._instance_lock:
                if not hasattr(BosThreadPool, "_instance"):
                    BosThreadPool._instance = object.__new__(cls)
                    logger.info("init bos_thread_pool")
                    BosThreadPool._instance._thread_pool = ThreadPool(BosftpCfg().parallel_upload_num)
                    BosThreadPool._instance._thread_num = BosftpCfg().parallel_upload_num
                    BosThreadPool._instance._semaphore = threading.Semaphore(BosftpCfg().parallel_upload_num)
        return BosThreadPool._instance

    def apply_async(self, func, args=(), kwds={}):
        logger.info("thread {0} acquire the semaphore and will aplly_async.".format(threading.currentThread().getName()))
        self._semaphore.acquire()
        self._thread_pool.apply_async(func=func, args=args, kwds=kwds, callback=self.release)

    def release(self, args):
        logger.info("thread {0} release the semaphore.".format(threading.currentThread().getName()))
        self._semaphore.release()

