# -*- coding: utf-8 -*-
"""
    Test for File Operation
"""
import os
import threading
import time
import uuid
import unittest
from socket import error as sock_error
from ftplib import FTP
from ftplib import FTP_TLS
from ftplib import error_temp
from ftplib import error_perm
import sys
sys.path.append("../bosftp")
import start_ftp_server 
from bosftp_config import BosftpCfg
from util import gen_file
from util import random_string
from util import get_file_md5
from util import storebinary
from util import retrbinary
from util import delete_file
from util import safe_remove_file
from util import get_size

HOST = "127.0.0.1"
USER = "xxx"
PASSWORD = "xxx"

class FileTest(unittest.TestCase):
    """
        FileTest
    """

    def setUp(self):
        self.ftp = FTP()
        self.ftp.connect(HOST, BosftpCfg().listen_port)
        self.ftp.login(USER, PASSWORD)

    def tearDown(self):
        self.ftp.quit()

    def test_upload(self):
        print "--------test_upload------\n"
        #size_list = [1024, 4*1024, 16*1024, 1024*1024, 16*1024*1024, 1024*1024*1024]
        MB = 1024 * 1024
        GB = 1024 * 1024 * 1024
        size_list = [1, 100, 1024, 4 * 1024, 16 * 1024, 1024 * 1024, 16 * MB]
        size_list += [1 * MB, 9 * MB, 11 * MB, 20 * MB, 100 * MB]
        size_list += [1 * GB, 3 * GB, 5 * GB, 7 * GB, 10 * GB]
        size_list += [5 * GB, 6 * GB, 8 * GB, 5 * GB, 5 * GB]
        up_file = "up_file"
        down_file = "down_file"
        for size in size_list:
            gen_file(up_file, size)
            _md5_up = get_file_md5(up_file)
            path = unicode(time.time())
            self.assertTrue(storebinary(self.ftp, up_file, path))

            self.assertTrue(retrbinary(self.ftp, path, down_file))
            _md5_down = get_file_md5(down_file) 
            self.assertEqual(_md5_up, _md5_down)
            self.assertTrue(delete_file(self.ftp, path))
        
        safe_remove_file(up_file)
        safe_remove_file(down_file)

    def test_get_size(self):
        print "--------test_get_size------\n"
        up_file = "up_file"
        size = 1024
        gen_file(up_file, size)
        path = unicode(time.time())
        self.assertTrue(storebinary(self.ftp, up_file, path))
        curr_size = get_size(self.ftp, path) 
        self.assertEqual(size, curr_size)
        self.assertTrue(delete_file(self.ftp, path))

        safe_remove_file(up_file)

    def test_rename(self):
        print "--------test_rename------\n"

        dest_name_list = ['中文', 'prefix_*&^% $#@!)(:?>', 
                          '中文1234&＊…………％＊&……＊&&％&……％&％＊&（！', 
                          random_string(1025), "none"]
        for dest_name in dest_name_list:
            path = unicode(time.time())
            if dest_name != "none": # don't upload file at this case
                up_file = "up_file"
                size = 1024
                gen_file(up_file, size)
                self.assertTrue(storebinary(self.ftp, up_file, path))
            try:
                self.ftp.rename(path, dest_name)
            except (error_temp, error_perm, sock_error) as e:
                if dest_name != "none" and len(dest_name) <= 1024:
                    print e
                    self.fail("test rename failed")
                elif dest_name == "none":
                    self.assertFalse(delete_file(self.ftp, dest_name))
                    self.assertFalse(delete_file(self.ftp, path))
                else:
                    self.assertTrue(delete_file(self.ftp, path))
                    self.assertFalse(delete_file(self.ftp, dest_name))
                continue
            # rename succeed
            self.assertFalse(delete_file(self.ftp, path))
           
            safe_remove_file(up_file)

    def test_chmod(self):
        print "--------test_chmod------\n"
        up_file = "up_file"
        size = 1024
        gen_file(up_file, size)
        path = unicode(time.time())
        self.assertTrue(storebinary(self.ftp, up_file, path))

        try:
            self.ftp.sendcmd('SITE CHMOD 666 %s' % path)
        except (error_temp, error_perm, sock_error) as e:
            self.assertTrue(e[0], "550 method chmod is not implied.")
        else:
            self.fail("test_chmod failed")
        self.assertTrue(delete_file(self.ftp, path))
    
    def test_file_name(self):
        print "--------test_file_name------\n"
        file_name_list = ['中文', 'prefix_*&^% $#@!)(:?>', '中文1234&＊…………％＊&……＊&&％&……％&％＊&（！']
        up_file = "up_file"
        gen_file(up_file, 4 * 1024)
        _md5_up = get_file_md5(up_file)
        down_file = "down_file"
        for name in file_name_list:
            self.assertTrue(storebinary(self.ftp, up_file, name))
            self.assertTrue(retrbinary(self.ftp, name, down_file))
            _md5_down = get_file_md5(down_file) 
            self.assertEqual(_md5_up, _md5_down)
            self.assertTrue(delete_file(self.ftp, name))

        safe_remove_file(up_file)
        safe_remove_file(down_file)

if __name__ == '__main__':
    unittest.main()
